LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
use ieee.std_logic_arith.all;
use ieee.std_logic_signed.all; 
use work.imagexlib_utils.all;

entity convert444to422_wrap is
generic
(
      num_taps                :  integer        := 3;          -- coefs must be an array of at least this length. Extra values in array are ignored.
      data_width              :  integer        := 8;
      coef_width              :  integer        := 3;          -- All integer coefficients must add up to 2^(coef_width - 1)
      pad_val                 :  integer        := 0;          -- Value used to fill pipeline at image edges - currently same for u and v
      pixel_drop              :  integer        := 0;          -- If 1, then filter should be optimised out
      coefs                   :  INTEGER_ARRAY  := (1, 2, 1)   -- Parameterised, but not dynamic.
);
 port
(
      clk                     :  in    std_logic;
      ce                      :  in    std_logic;
      din_valid               :  in    std_logic;
      vsync_in                :  in    std_logic;
      hsync_in                :  in    std_logic;
      vblank_in               :  in    std_logic;
      hblank_in               :  in    std_logic;
      luma_in                 :  in    std_logic_vector((data_width-1) downto 0); -- Brought in for latency equalisation
      cr_in                   :  in    std_logic_vector((data_width-1) downto 0);
      cb_in                   :  in    std_logic_vector((data_width-1) downto 0);
      chroma_out              :  out   std_logic_vector((data_width-1) downto 0); -- Multiplexed Cr, Cb
      luma_out                :  out   std_logic_vector((data_width-1) downto 0); -- Latency equalised luma output
      vsync_out               :  out   std_logic;
      hsync_out               :  out   std_logic;
      vblank_out              :  out   std_logic;
      hblank_out              :  out   std_logic;
      saturate                :  out   std_logic;
      sat_type                :  out   std_logic;
      dout_valid              :  out   std_logic                                    -- Depends upon latency through system
    
);
end convert444to422_wrap;

architecture rtl of convert444to422_wrap is

signal   left_cr_pad                :  std_logic_vector((data_width - 1) downto 0);
signal   left_cb_pad                :  std_logic_vector((data_width - 1) downto 0);
signal   right_cr_pad               :  std_logic_vector((data_width - 1) downto 0);
signal   right_cb_pad               :  std_logic_vector((data_width - 1) downto 0);
type     DelayedVChanType           is array (31 downto  0) of std_logic_vector((data_width - 1) downto 0);
signal   DelayedCrIn                :  DelayedVChanType;
signal   DelayedCbIn                :  DelayedVChanType;
signal   DelayedYIn                 :  DelayedVChanType;

signal   Delayed_din_valid          :  std_logic_vector(31 downto 0);
signal   Delayed_vsync              :  std_logic_vector(31 downto 0);
signal   Delayed_hsync              :  std_logic_vector(31 downto 0);
signal   Delayed_vblank             :  std_logic_vector(31 downto 0);
signal   Delayed_hblank             :  std_logic_vector(31 downto 0);
signal   d_cr_in                    :  std_logic_vector((data_width - 1) downto 0);
signal   d_cb_in                    :  std_logic_vector((data_width - 1) downto 0);
signal   d_y_in                     :  std_logic_vector((data_width - 1) downto 0);
signal   d_din_valid                :  std_logic;

begin

proc1 : process (clk)
begin
if (clk'event) and (clk =  '1') then
   if   (ce = '1') then
      DelayedCrIn(0)                   <= cr_in;
      DelayedCrIn(31 downto   1)       <= DelayedCrIn(30 downto 0); -- sufficiently big for future generic case - should be optimised out.

      DelayedCbIn(0)                   <= cb_in;
      DelayedCbIn(31 downto 1)         <= DelayedCbIn(30 downto 0); -- sufficiently big for future generic case - should be optimised out.

      Delayed_din_valid(0)             <= din_valid;
      Delayed_din_valid(31 downto 1)   <= Delayed_din_valid(30 downto 0); -- sufficiently big for future generic case - should be optimised out.
      d_din_valid                      <= Delayed_din_valid(0) or Delayed_din_valid(4);   
                                                   -- din_valid must be lengthened by a period equal to the length of the filter.

      DelayedYIn(0)                    <= luma_in;
      DelayedYIn(31  downto 1)         <= DelayedYIn(30 downto 0); -- sufficiently big for future generic case - should be optimised out.
      d_y_in                           <= DelayedYIn(2); -- Tap off delay line at a point equal to half the filter size.

      Delayed_vsync(0)                 <= vsync_in;
      Delayed_vsync(31 downto 1)       <= Delayed_vsync(30 downto 0); -- sufficiently big for future generic case - should be optimised out.
      vsync_out                        <= Delayed_vsync(7); 

      Delayed_hsync(0)                 <= hsync_in;
      Delayed_hsync(31 downto 1)       <= Delayed_hsync(30 downto 0); -- sufficiently big for future generic case - should be optimised out.
      hsync_out                        <= Delayed_hsync(7); 

      Delayed_vblank(0)                <= vblank_in;
      Delayed_vblank(31 downto 1)      <= Delayed_vblank(30 downto 0); -- sufficiently big for future generic case - should be optimised out.
      vblank_out                       <= Delayed_vblank(7); 

      Delayed_hblank(0)                <= hblank_in;
      Delayed_hblank(31 downto 1)      <= Delayed_hblank(30 downto 0); -- sufficiently big for future generic case - should be optimised out.
      hblank_out                       <= Delayed_hblank(7); 

      dout_valid                       <= Delayed_din_valid(10);

      -- Detect rising (left) edge of din_valid and use Cr and Cb input values to pad filter
      -- before first data on current line enters the filter
      if (din_valid = '1') and (Delayed_din_valid(0) = '0') then
         left_cr_pad                   <= cr_in;
         left_cb_pad                   <= cb_in;
      end if;
      
      -- Detect falling (right) edge of din_valid and use Cr and Cb input values to pad filter
      -- after last data on current line enters filter
      if (din_valid = '0') and (Delayed_din_valid(0) = '1') then
         right_cr_pad                  <= DelayedCrIn(0);
         right_cb_pad                  <= DelayedCbIn(0);
      end if;
       
      -- Mux delayed input data with padding values
      if (Delayed_din_valid(0)  =  '1') and (Delayed_din_valid(3) = '0') then
         --   Left edge
         d_cr_in                       <= left_cr_pad;
         d_cb_in                       <= left_cb_pad;
      elsif (Delayed_din_valid(2) = '0') and (Delayed_din_valid(4) = '1')  then
         --   Right edge
         d_cr_in                       <= right_cr_pad;
         d_cb_in                       <= right_cb_pad;
      elsif (Delayed_din_valid(0) = '0') and (Delayed_din_valid(2) = '0') then
         d_cr_in                       <= (others => '0');
         d_cb_in                       <= (others => '0');
      else
          -- Everywhere else
         d_cr_in                       <= DelayedCrIn(2);
         d_cb_in                       <= DelayedCbIn(2);
      end   if;
   
   end if;
   
end if;
end process;

u0 : entity work.convert444to422
generic  map
(
    num_taps         => num_taps,
    data_width       => data_width, 
    coef_width       => coef_width, 
    pad_val          => pad_val,
    pixel_drop       => pixel_drop, 
    coefs            => coefs     
)
port map
(                          
   clk               => clk,
   din_valid         => d_din_valid, -- Select correct delay tap according to filter size
   luma_in           => d_y_in,
   cr_in             => d_cr_in,
   cb_in             => d_cb_in,
   ce                => ce,
   chroma_out        => chroma_out,
   luma_out          => luma_out,
   saturate          => saturate,
   sat_type          => sat_type,
   dout_valid        => open
);

end   rtl;