------------------------------------------------------------------
--      _____
--     /     \
--    /____   \____
--   / \===\   \==/
--  /___\===\___\/  AVNET
--       \======/
--        \====/    
-----------------------------------------------------------------
--
-- This design is the property of Avnet.  Publication of this
-- design is not authorized without written consent from Avnet.
-- 
-- Please direct any questions to:  technical.support@avnet.com
--
-- Disclaimer:
--    Avnet, Inc. makes no warranty for the use of this code or design.
--    This code is provided  "As Is". Avnet, Inc assumes no responsibility for
--    any errors, which may appear in this code, nor does it make a commitment
--    to update the information contained herein. Avnet, Inc specifically
--    disclaims any implied warranties of fitness for a particular purpose.
--                     Copyright(c) 2010 Avnet, Inc.
--                             All rights reserved.
--
------------------------------------------------------------------
--
-- Create Date:         Apr 13, 2010
-- Design Name:         Color Space Converter - RGB to YCrCb 4:2:2
-- Module Name:         csc_rgb_to_ycrcb422.vhd
-- Project Name:        IVK
-- Target Devices:      Spartan-6
-- Avnet Boards:        IVK
--
-- Tool versions:       ISE 11.4
--
-- Description:         This module performs two conversion steps:
--                      1) Color Space Conversion
--                         - RGB to YCrCB 4:4:4
--                         - based on XAPP930
--                      2) Chroma Resampling
--                         - YCrCb 4:4:4 to YCrCb 4:2:2
--                         - based on XAPP932
--
--                      The Video I/O for this module are:
--                      - Video Input  = 24 bit RGB
--                      - Video Output = 16 bit YCrCb 4:2:2
--
-- References:        
--    http://www.xilinx.com/support/documentation/application_notes/xapp930.pdf
--    http://www.xilinx.com/support/documentation/application_notes/xapp931.pdf
--    http://www.xilinx.com/support/documentation/application_notes/xapp932.pdf
--                      
-- Dependencies:        
--
-- Revision:            Apr 13, 2010: 1.01 Initial version
--
------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

use WORK.ImageXlib_utils.all;

entity csc_rgb_to_ycrcb422 is
   port (
      clk               : in  STD_LOGIC;
      ce                : in  STD_LOGIC;

      -- XSVI Input (24 bit RGB)
      vblank_i          : in  STD_LOGIC;
      hblank_i          : in  STD_LOGIC;
      vsync_i           : in  STD_LOGIC;
      hsync_i           : in  STD_LOGIC;
      active_video_i    : in  STD_LOGIC;
      video_data_i      : in  STD_LOGIC_VECTOR (23 downto 0);

      -- XSVI Input (16 bit YCrCb 4:2:2)
      vblank_o          : out STD_LOGIC;
      hblank_o          : out STD_LOGIC;
      vsync_o           : out STD_LOGIC;
      hsync_o           : out STD_LOGIC;
      active_video_o    : out STD_LOGIC;
      video_data_o      : out STD_LOGIC_VECTOR (15 downto 0);

      debug_o           : out STD_LOGIC_VECTOR (26 downto 0)
           
    );
end csc_rgb_to_ycrcb422;


architecture rtl of csc_rgb_to_ycrcb422 is

   signal vblank_d      : STD_LOGIC;
   signal hblank_d      : STD_LOGIC;
   signal vsync_d       : STD_LOGIC;
   signal hsync_d       : STD_LOGIC;
   signal active_video_d: STD_LOGIC;
   signal video_red_d   : STD_LOGIC_VECTOR (7 downto 0);
   signal video_green_d : STD_LOGIC_VECTOR (7 downto 0);
   signal video_blue_d  : STD_LOGIC_VECTOR (7 downto 0);

   --
   -- Color Space Conversion (24 bit RGB to 24 bit YCrCb)
   --

   component Xil_RGB2YCrCb
   generic 
   (
      FAMILY_HAS_MAC             :  integer  := 1;
      FABRIC_ADDS                :  integer  := 1; -- Adders are implemented using logic fabric based adders
      IWIDTH                     :  integer  := 9;
      CWIDTH                     :  integer  := 17; -- Coefficients are unsigned
      MWIDTH                     :  integer  := 17;   -- Width of (B' - Y'_601) and (R' - Y'_601) data (data width at CCOEF and DCOEF multipliers)
      OWIDTH                     :  integer  := 9;
      YMAX                       :  integer  := 255;
      YMIN                       :  integer  := 0;
      CMAX                       :  integer  := 255;
      CMIN                       :  integer  := 0;
      YOFFSET                    :  integer  := 0;
      COFFSET                    :  integer  := 128;    -- represent 0.5 in 9.7 format
      ACOEF                      :  integer  := 613;    -- 0.29931
      BCOEF                      :  integer  := 233;    -- 0.11376953125
      CCOEF                      :  integer  := 1460;   -- 0.712890625
      DCOEF                      :  integer  := 1155;   -- 0.56396484375
      HAS_CLIP                   :  integer  := 1; 
      HAS_CLAMP                  :  integer  := 1
   ); 
   port
   ( 
      Y                          :  out   std_logic_vector(OWIDTH-1 downto 0);  -- Y  = a(R-G) + G + b(B-G)
      Cr                         :  out   std_logic_vector(OWIDTH-1 downto 0);  -- Cr = d(R-Y)  
      Cb                         :  out   std_logic_vector(OWIDTH-1 downto 0);  -- Cb = c(B-Y)
      R                          :  in    std_logic_vector(IWIDTH-1 downto 0);   
      G                          :  in    std_logic_vector(IWIDTH-1 downto 0);
      B                          :  in    std_logic_vector(IWIDTH-1 downto 0);
      V_BLANK_in                 :  in    std_logic := '0';
      H_BLANK_in                 :  in    std_logic := '0';
      V_SYNC_in                  :  in    std_logic := '0';
      H_SYNC_in                  :  in    std_logic := '0';
      PIX_EN_in                  :  in    std_logic := '1';
      V_BLANK_out                :  out   std_logic;
      H_BLANK_out                :  out   std_logic;
      V_SYNC_out                 :  out   std_logic;
      H_SYNC_out                 :  out   std_logic;
      PIX_EN_out                 :  out   std_logic;
      clk                        :  in    std_logic;
      ce                         :  in    std_logic := '1';
      sclr                       :  in    std_logic := '0'
   );
   end component;

   signal   t_Y444      :  std_logic_vector(7 downto 0);
   signal   t_Cr444     :  std_logic_vector(7 downto 0);
   signal   t_Cb444     :  std_logic_vector(7 downto 0);
   signal   t_de_o      :  std_logic;
   signal   t_vsync_o   :  std_logic;
   signal   t_hsync_o   :  std_logic;
   signal   t_vblank_o  :  std_logic;
   signal   t_hblank_o  :  std_logic;

   --
   -- Chroma Re-Sampling (24 bit YCrCb 4:4:4 to 16 bit YCrCb 4:2:2)
   --

   component convert444to422_wrap
   generic 
   (                           
      num_taps                   :  integer  := 3;                   -- coefs must be an array of at least this length. Extra values in array are ignored.
      data_width                 :  integer  := 8;
      coef_width                 :  integer  := 3;                   -- All integer coefficients must add up to 2^(coef_width - 1)
      pad_val                    :  integer  := 0;                   -- Value used to fill pipeline at image edges - currently same for u and v
      pixel_drop                 :  integer  := 0;                   -- If 1, then filter should be optimised out
      coefs	                     :  INTEGER_ARRAY  := (1, 2, 1)      -- Parameterised, but not dynamic.            
   );
   port 
   (                              
      clk                        :  in    std_logic;
      ce                         :  in    std_logic;
      din_valid                  :  in    std_logic;
      vblank_in                  :  in    std_logic;
      hblank_in                  :  in    std_logic;
      vsync_in                   :  in    std_logic;
      hsync_in                   :  in    std_logic;
      luma_in                    :  in    std_logic_vector((data_width-1) downto 0);
      cr_in                      :  in    std_logic_vector((data_width-1) downto 0);  
      cb_in                      :  in    std_logic_vector((data_width-1) downto 0);  
      chroma_out                 :  out   std_logic_vector((data_width-1) downto 0);   -- Multiplexed Cr, Cb
      luma_out                   :  out   std_logic_vector((data_width-1) downto 0);   -- Latency equalised luma output
      saturate                   :  out   std_logic;
      sat_type                   :  out   std_logic;
      vblank_out                 :  out   std_logic;
      hblank_out                 :  out   std_logic;
      vsync_out                  :  out   std_logic;
      hsync_out                  :  out   std_logic;
      dout_valid                 :  out   std_logic 
   );
   end component;

   signal   Y_444to422      :  std_logic_vector(7 downto 0);
   signal   CrCb_444to422   :  std_logic_vector(7 downto 0);
   signal   vblank_444to422 :  std_logic;
   signal   hblank_444to422 :  std_logic;
   signal   vsync_444to422  :  std_logic;
   signal   hsync_444to422  :  std_logic;
   signal   de_444to422     :  std_logic;
   

begin

   --
   -- Input Registers
   --

   In_Reg : process (clk)
   begin
      if Rising_Edge(clk) then
         vblank_d       <= vblank_i;
         hblank_d       <= hblank_i;
         vsync_d        <= vsync_i;
         hsync_d        <= hsync_i;
         active_video_d <= active_video_i;
         video_red_d    <= video_data_i(23 downto 16);
         video_green_d  <= video_data_i( 7 downto  0);
         video_blue_d   <= video_data_i(15 downto  8);
      end if;
   end process;

   --
   -- Color Space Conversion (24 bit RGB to 24 bit YCrCb)
   --

   RGB2YCrCb: Xil_RGB2YCrCb
   generic map 
   (
      FAMILY_HAS_MAC => 1,
      FABRIC_ADDS    => 1,      -- Adders are implemented using logic fabric based adders
      IWIDTH         => 8,
      CWIDTH         => 17,     -- Coefficients are unsigned
      MWIDTH         => 18,     -- Width of (B' - Y'_601) and (R' - Y'_601) data (data width at CCOEF and DCOEF multipliers)
      OWIDTH         => 8,
      YMAX           => 240,
      YMIN           => 16,
      CMAX           => 240,
      CMIN           => 16,
      YOFFSET        => 16,
      COFFSET        => 128,    -- represent 0.5 in 9.7 format
      ACOEF          => 19595,  -- CA=0.299    ACOEF=CA*(2^(CWIDTH-1))
      BCOEF          => 7471,   -- CB=0.114    BCOEF=CB*(2^(CWIDTH-1))
      CCOEF          => 36962,  -- CC=0.492111 CCOEF=CC*(2^(CWIDTH-1))
      DCOEF          => 46727,  -- 57494,  -- CD=0.877283 DCOEF=CD*(2^(CWIDTH-1))
      HAS_CLIP       => 1, 
      HAS_CLAMP      => 1
   )
--   ( 
--      FAMILY_HAS_MAC => 1,
--      FABRIC_ADDS    => 1,      -- Adders are implemented using logic fabric based adders
--      IWIDTH         => 8,
--      CWIDTH         => 17,     -- Coefficients are unsigned
--      MWIDTH         => 18,     -- Width of (B' - Y'_601) and (R' - Y'_601) data (data width at CCOEF and DCOEF multipliers)
--      OWIDTH         => 8,
--      YMAX           => 255,
--      YMIN           => 0,
--      CMAX           => 255,
--      CMIN           => 0,
--      YOFFSET        => 16,
--      COFFSET        => 128,    -- represent 0.5 in 9.7 format
--      ACOEF          => 19595,  -- CA=0.299    ACOEF=CA*(2^(CWIDTH-1))
--      BCOEF          => 7471,   -- CB=0.114    BCOEF=CB*(2^(CWIDTH-1))
--      CCOEF          => 32251,  -- CC=0.492111 CCOEF=CC*(2^(CWIDTH-1))
--      DCOEF          => 57490,  -- 57494,  -- CD=0.877283 DCOEF=CD*(2^(CWIDTH-1))
--      HAS_CLIP       => 1, 
--      HAS_CLAMP      => 1
--   )
   port map 
   (
      Y              => t_Y444,
      Cr             => t_Cr444,
      Cb             => t_Cb444,
      R              => video_red_d,   
      G              => video_green_d,
      B              => video_blue_d,
      V_SYNC_in      => vsync_d,
      H_SYNC_in      => hsync_d,
      PIX_EN_in      => active_video_d,
      V_SYNC_out     => t_vsync_o,
      H_SYNC_out     => t_hsync_o,
      PIX_EN_out     => t_de_o,
      clk            => clk,
      ce             => ce,
      sclr           => '0'
   );

   --
   -- Chroma Re-Sampling (24 bit YCrCb 4:4:4 to 16 bit YCrCb 4:2:2)
   --

   YCrCb444to422: convert444to422_wrap
   generic map 
   (
      num_taps       => 3,                   -- coefs must be an array of at least this length. Extra values in array are ignored.
      data_width     => 8,
      coef_width     => 3,                   -- All integer coefficients must add up to 2^(coef_width - 1)
      pad_val        => 0,                   -- Value used to fill pipeline at image edges - currently same for u and v
      pixel_drop     => 0,                   -- If 1, then filter should be optimised out
      coefs	         => (1, 2, 1)            -- Parameterised, but not dynamic.            
   )
   port map 
   (
      clk            => clk,
      ce             => '1',
      din_valid      => t_de_o,
      vblank_in      => t_vblank_o,
      hblank_in      => t_hblank_o,
      vsync_in       => t_vsync_o,
      hsync_in       => t_hsync_o,
      luma_in        => t_Y444,
      cr_in          => t_Cr444,
      cb_in          => t_Cb444,
      chroma_out     => CrCb_444to422,       -- Multiplexed Cr, Cb
      luma_out       => Y_444to422,          -- Latency equalised luma output
      saturate       => open,
      sat_type       => open,
      vblank_out     => vblank_444to422,
      hblank_out     => hblank_444to422,
      vsync_out      => vsync_444to422,
      hsync_out      => hsync_444to422,
      dout_valid     => de_444to422
   );


   --
   -- Output Registers
   --

   Out_Reg : process (clk)
   begin
      if Rising_Edge(clk) then
         vblank_o          <= vblank_444to422;
         hblank_o          <= hblank_444to422;
         vsync_o           <= vsync_444to422;
         hsync_o           <= hsync_444to422;
         active_video_o    <= de_444to422;
         video_data_o      <= CrCb_444to422 & Y_444to422;
      end if;
   end process;

   --
   -- Debug Port
   --

   Debug_Port : process (clk)
   begin
      if Rising_Edge(clk) then
         debug_o  <= t_de_o & t_vsync_o & t_hsync_o & t_Y444 & t_Cr444 & t_Cb444;
      end if;
   end process;

end rtl;
