-------------------------------------------------------------------------------
-- Copyright (c) 2005 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
-------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   
-- \   \   \/    Vendor: Xilinx
--  \   \        Filename: convert444to422.vhd
--  /   /        Date Created: Sept 26 2005
-- /___/   /\    
-- \   \  /  \
--  \___\/\___\
-- 
--Device: Any Xilinx family.
--Purpose: VHDL module for converting chroma video in 4:2:2 format to 4:4:4. 
--Reference:
--    XAPP932
--Revision History:
-------------------------------------------------------------------------------
--
-- $RCSfile: convert444to422.vhd,v $ $Revision: 1.13 $ $Date: 2006/04/18 04:39:11 $
--
-- ************************************************************************
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
use ieee.std_logic_arith.all;
use ieee.std_logic_signed.all; 
use work.imagexlib_utils.all;

entity convert444to422 is
   generic
   (
      num_taps       :  integer  := 3;                   -- coefs must be an array of at least this length. Extra values in array are ignored.
      data_width     :  integer  := 8;
      coef_width     :  integer  := 3;                   -- All integer coefficients must add up to 2^(coef_width - 1)
      pad_val        :  integer  := 0;                   -- Value used to fill pipeline at image edges - currently same for u and v
      pixel_drop     :  integer  := 0;                   -- If 1, then filter should be optimised out
      coefs	         :  INTEGER_ARRAY  := (1, 2, 1)  -- Parameterised, but not dynamic.            
   );
   port
	(
      clk               :  in    std_logic;
      ce                :  in    std_logic;
      din_valid         :  in    std_logic;
      luma_in           :  in    std_logic_vector((data_width-1) downto 0);   -- Brought in for latency equalisation
      cr_in             :  in    std_logic_vector((data_width-1) downto 0);  
      cb_in             :  in    std_logic_vector((data_width-1) downto 0);  
      chroma_out        :  out   std_logic_vector((data_width-1) downto 0);   -- Multiplexed Cr, Cb
      luma_out          :  out   std_logic_vector((data_width-1) downto 0);   -- Latency equalised luma output
      saturate          :  out   std_logic;
      sat_type          :  out   std_logic;
      dout_valid        :  out   std_logic                                    -- Depends upon latency through system

   );
end convert444to422;


architecture rtl of convert444to422 is

constant num_taps_div2                    :  integer  := num_taps/2;

-- Here we ensure that the number of taps implemented is an odd number - this is better for this architecture
constant numtaps_made_odd                 :  integer  := (num_taps_div2 * 2) + 1;

-- This brings the Boolean nature of pixel_drop into the latency calculation.
constant latency                          :  integer  := ((1-pixel_drop)*(numtaps_made_odd+5)) + (pixel_drop * 2);

constant num_taps_div2_vec                :  std_logic_vector(0 downto 0) := conv_std_logic_vector(num_taps_div2, 1);
constant outer_taps_are_on_odd_samples    :  integer  := conv_integer('0' & num_taps_div2_vec); -- IMPORTANT : 0 = false, 1 = true
constant num_even_h_taps                  :  integer  := (numtaps_made_odd+(1-outer_taps_are_on_odd_samples))/2;      -- Rounds down to the integer value
constant num_odd_h_taps                   :  integer  := (numtaps_made_odd+(outer_taps_are_on_odd_samples))/2;    -- Rounds down to the integer value
signal   logic1                           :  std_logic   := '1';
signal   logic0                           :  std_logic   := '0';

type     coefs_array_type                 is array (0 to num_taps) of std_logic_vector((coef_width - 1) downto 0);
signal   coefs_array                      :  coefs_array_type;
signal   d_CrCbMuxSel                     :  std_logic;
signal   CrCbEven, CrCbOdd                :  std_logic_vector((data_width - 1) downto 0);
signal   d_CrCbEven, d2_CrCbEven          :  std_logic_vector((data_width - 1) downto 0);
signal   delayed_CrCbEven                 :  std_logic_vector((data_width - 1) downto 0);
signal   d_CrCbOdd, d2_CrCbOdd            :  std_logic_vector((data_width - 1) downto 0);
signal   delayed_CrCbOdd                  :  std_logic_vector((data_width - 1) downto 0);

signal   chroma_even_hpipe_out_long       :  std_logic_vector_array((num_even_h_taps - 1) downto 0);
type     chroma_even_hpipe_out_h_type     is array((num_even_h_taps - 1) downto 0) of std_logic_vector((data_width - 1) downto 0);
signal   chroma_even_hpipe_out            :  chroma_even_hpipe_out_h_type;

signal   chroma_odd_hpipe_out_long        :  std_logic_vector_array((num_odd_h_taps - 1) downto 0);
type     chroma_odd_hpipe_out_h_type      is array((num_odd_h_taps - 1) downto 0) of std_logic_vector((data_width - 1) downto 0);
signal   chroma_odd_hpipe_out             :  chroma_odd_hpipe_out_h_type;

type     chroma_even_mult_out_h_type      is array((num_even_h_taps - 1) downto 0) of std_logic_vector((coef_width + data_width) downto 0);
signal   chroma_even_mult_out             :  chroma_even_mult_out_h_type;

type     chroma_odd_mult_out_h_type       is array((num_odd_h_taps - 1) downto 0) of std_logic_vector((coef_width + data_width) downto 0);
signal   chroma_odd_mult_out              :  chroma_odd_mult_out_h_type;

type     chroma_even_mult_out_int_h_type  is array((num_even_h_taps - 1) downto 0) of integer;
signal   chroma_even_mult_out_int         :  chroma_even_mult_out_int_h_type;

type     chroma_odd_mult_out_int_h_type   is array((num_odd_h_taps - 1) downto 0) of integer;
signal   chroma_odd_mult_out_int          :  chroma_odd_mult_out_int_h_type;

signal   even_partial_sum_int             :  chroma_even_mult_out_int_h_type;
signal   odd_partial_sum_int              :  chroma_odd_mult_out_int_h_type;
signal   h_sum_int                        :  integer;

type     d_luma_type                      is array (0 to (latency - 1)) of std_logic_vector((data_width - 1) downto 0);
signal   d_luma                           :  d_luma_type;
signal   d_data_valid                     :  std_logic_vector((latency - 1) downto 0);
signal   d_cr_in, d2_cr_in                :  std_logic_vector((data_width - 1) downto 0);
signal   d_cb_in, d2_cb_in                :  std_logic_vector((data_width - 1) downto 0);

constant full_precision_width             :  integer := data_width + coef_width + LOG2_BASE(num_taps) + 1;
signal   chroma_out_int_full_precision    :  integer;
signal   chroma_out_full_precision        :  std_logic_vector((full_precision_width - 1) downto 0);
signal   rounding_offset                  :  integer := 2**(coef_width - 2);
signal   vs_out                                    :  std_logic;
begin

-- Coefs are fed in from GUI as integers. Convert them to std_logic_vectors here.
convert_coefs : for i in 0 to (num_taps - 1) generate
   coefs_array(i)    <= conv_std_logic_vector(coefs(i), coef_width);
end generate;
coefs_array(num_taps) <= (others => '0');

IO_Registers : process(clk)
begin
   if (clk'event and clk = '1') then
   -- Register Luma_in, luma_out, chroma_in and data-valid signals - this counts as first and last delays.         
      if (ce = '1') then 
         d_luma(0)         <= luma_in;
         luma_out          <= d_luma(latency - 2);   
         d_data_valid(0)   <= din_valid;
         dout_valid        <= d_data_valid(latency - 2);
         d_cr_in           <= cr_in;
         d2_cr_in          <= d_cr_in;
         d_cb_in           <= cb_in;
         d2_cb_in          <= d_cb_in;  

         d_CrCbEven        <= CrCbEven;
         d2_CrCbEven       <= d_CrCbEven;      
         d_CrCbOdd         <= CrCbOdd;
         d2_CrCbOdd        <= d_CrCbOdd;         
  
         -- Delay Luma and data-valid signal
         compensate_delay : for i in 1 to (latency - 2) loop
            d_luma(i)         <= d_luma(i-1);
            d_data_valid(i)   <= d_data_valid(i-1);
         end loop;
      end if;
   end if;
end process;      


-- Now we take advantage of the chroma decimation.
-- For each of Cr and Cb channels, we are decimating by 2 horizontally.
-- Hence we can assume some resource sharing between Cr and Cb channels here and maintain
-- full clock rate on the multilpiers.
-- Question is - where does the CrCb MUX happen.
-- Conceptually, the most obvious thing to do is delay cb by one clock, and MUX the inputs to each multiplier
-- However, this implies the duplication of the horizontal pipe.
-- A better solution is to mux before the pipe, provided the pipe can handle the interleaved data.
-- First we create a mux selector signal which is based upon the first valid input to the block in a line.
CrCbMUX : process (clk)
begin
   if (clk'event) and (clk = '1') then
      if (ce = '1') then
         if (d_data_valid(0) = '0') then
            d_CrCbMuxSel  <= '0';
         else
            d_CrCbMuxSel  <= not(d_CrCbMuxSel);
         end if;       
   
         -- Here is the MUX that selects Cr and Cb for odd and even pipes (please read comment above).
         -- Mux in the padding value.
         if (d_data_valid(0) = '0') then
            CrCbEven    <= conv_std_logic_vector(pad_val, data_width);
         else
            -- Now here is the even data mux
            if (d_CrCbMuxSel = '0') then
               CrCbEven    <= d_cb_in;
            else
               CrCbEven    <= d2_cr_in;
            end if;
         end if;
         
         -- Mux in the padding value.
         if (d_data_valid(0) = '0') then
            CrCbOdd     <= conv_std_logic_vector(pad_val, data_width);      
         else
            -- Now here is the odd data mux
            if (d_CrCbMuxSel = '1') then
               CrCbOdd     <= d_cb_in;
            else
               CrCbOdd     <= d2_cr_in;
            end if;
         end if;
      end if;
   end if;
end process;  

-- Define delayed_CrCbEven accoding to the size of the filter:
-- Even pipe requires delaying by one CrCb cycle (2 clk cycles) relative to odd in order to align the pipes and their outputs.
-- However, if num_even_taps > num_odd_taps (ie outer_taps_are_on_odd_samples = 0) then this automatically brings in the additional cycle.
-- Conversely if num_even_taps < num_odd_taps, we need a delay of 2 cycles on the even pipe.
delayed_CrCbEven  <= CrCbEven when (outer_taps_are_on_odd_samples = 0) else d2_CrCbEven;-- eg 3-taps - delay even
delayed_CrCbOdd   <= CrCbOdd  when (outer_taps_are_on_odd_samples = 1) else d2_CrCbOdd; -- eg 3-taps - don't delay odd

-- Here we create one pipe for each of the odd and even paths, with Cr and Cb interleaved
even_chroma_hpipe : entity work.mc_h_pipe 
generic map
(
   num_channels   => 2,
   num_h_taps     => num_even_h_taps,
   data_width     => data_width
)
port map
(
   ce             => ce,
   clk            => clk,
   data_in        => delayed_CrCbEven,
   shift_en       => logic1,
   data_out       => chroma_even_hpipe_out_long
);

shorten_even_h_pipe : for j in 0 to (num_even_h_taps - 1) generate
   chroma_even_hpipe_out(j)   <= chroma_even_hpipe_out_long(j)((data_width - 1) downto 0);
end generate;

odd_chroma_hpipe : entity work.mc_h_pipe 
generic map
(
   num_channels   => 2,
   num_h_taps     => num_odd_h_taps,
   data_width     => data_width
)
port map
(
   ce             => ce,
   clk            => clk,
   data_in        => delayed_CrCbOdd,
   shift_en       => logic1,
   data_out       => chroma_odd_hpipe_out_long
);

shorten_odd_h_pipe : for j in 0 to (num_odd_h_taps - 1) generate
   chroma_odd_hpipe_out(j)   <= chroma_odd_hpipe_out_long(j)((data_width - 1) downto 0);   
end generate;

mult_add_loops : process (clk)
begin
   if (clk'event) and (clk = '1') then
      if (ce = '1') then   
         -- Multiply even half
         mult_h_even_loop : for j in 0 to (num_even_h_taps - 1) loop
            chroma_even_mult_out(j) <= ('0' & chroma_even_hpipe_out(j)) * coefs_array((2*j)+outer_taps_are_on_odd_samples);
         end loop;
   
         -- Add even multiplier results in chain
         even_partial_sum_int(0)   <= chroma_even_mult_out_int(0);   
         sum_h_even_loop : for j in 1 to (num_even_h_taps - 1) loop
            even_partial_sum_int(j) <= even_partial_sum_int(j-1) + chroma_even_mult_out_int(j);
         end loop;
   
         -- Multiply odd half
         mult_h_odd_loop : for j in 0 to (num_odd_h_taps - 1) loop
            chroma_odd_mult_out(j) <= ('0' & chroma_odd_hpipe_out(j)) * coefs_array((2*j)+(1-outer_taps_are_on_odd_samples));
         end loop;
   
         -- Add odd multiplier results in chain
         odd_partial_sum_int(0)   <= chroma_odd_mult_out_int(0);   
         sum_h_odd_loop : for j in 1 to (num_odd_h_taps - 1) loop
            odd_partial_sum_int(j) <= odd_partial_sum_int(j-1) + chroma_odd_mult_out_int(j);
         end loop;
   
         -- Now sum the odd and even halves (for each vertical tap)
         h_sum_int                     <= even_partial_sum_int(num_even_h_taps - 1) + odd_partial_sum_int(num_odd_h_taps - 1);         
         chroma_out_int_full_precision <= h_sum_int + rounding_offset;
      end if;
   end if;
end process;   

-- Asynchronous assignments
-- Prior to adding the multiply results, convert all mult out values to integers.
convert_mult_out_h_even_loop : for j in 0 to (num_even_h_taps - 1) generate   
--   chroma_even_mult_out_int(j)      <= conv_integer('0' & chroma_even_mult_out(j));         
   chroma_even_mult_out_int(j)      <= conv_integer(chroma_even_mult_out(j));         
end generate;
convert_mult_out_h_odd_loop : for j in 0 to (num_odd_h_taps - 1) generate   
--   chroma_odd_mult_out_int(j)       <= conv_integer('0' & chroma_odd_mult_out(j));
   chroma_odd_mult_out_int(j)       <= conv_integer(chroma_odd_mult_out(j));
end generate;

chroma_out_full_precision     <= conv_std_logic_vector(chroma_out_int_full_precision, full_precision_width);

-- Now bring in the pixel_drop option - this will optimise the entire filter out when set to 1.
chroma_out  <= chroma_out_full_precision((coef_width + data_width-2) downto (coef_width-1)) when pixel_drop = 0 else CrCbEven;

saturate <= chroma_out_full_precision(coef_width + data_width-1);
sat_type <= chroma_out_full_precision(coef_width + data_width);

end rtl;