-------------------------------------------------------------------------------
-- Copyright (c) 2005 Xilinx, Inc.
-- This design is confidential and proprietary of Xilinx, All Rights Reserved.
-------------------------------------------------------------------------------
--   ____  ____
--  /   /\/   /
-- /___/  \  /   
-- \   \   \/    Vendor: Xilinx
--  \   \        Filename: convert422to444.vhd
--  /   /        Date Created: Sept 26 2005
-- /___/   /\    
-- \   \  /  \
--  \___\/\___\
-- 
--Device: Any Xilinx family.
--Purpose: VHDL module for converting chroma video in 4:2:2 format to 4:4:4. 
--Reference:
--    XAPP932
--Revision History:
-------------------------------------------------------------------------------
--
-- $RCSfile: convert422to444.vhd,v $ $Revision: 1.21 $ $Date: 2006/04/18 04:39:10 $
--
-- ************************************************************************
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
use ieee.std_logic_arith.all;
use ieee.std_logic_signed.all; 
use work.ImageXlib_utils.all;

entity convert422to444 is
   generic
   (
      num_taps       :  integer  := 2;                -- coefs must be an array of at least this length. Extra values in array are ignored.
      data_width     :  integer  := 8;
      coef_width     :  integer  := 2;                -- All integer coefficients must add up to 2^(coef_width - 1)
      pad_val        :  integer  := 0;                -- Value used to fill pipeline at image edges - currently same for cr and cb
      replicate      :  integer  := 0;                -- If 1, then filter should be optimised out
      coefs	         :	INTEGER_ARRAY := (1, 1)   -- Parameterised, but not dynamic.
   );
   port
	(
	   ce              :  in    std_logic;
	   clk               :  in    std_logic;
      din_valid         :  in    std_logic;
      luma_in           :  in    std_logic_vector((data_width-1) downto 0);   -- Brought in for latency equalisation
      chroma_in         :  in    std_logic_vector((data_width-1) downto 0);   -- Multiplexed cr, cb
      luma_out          :  out   std_logic_vector((data_width-1) downto 0);   -- Latency equalised luma output
      cr_out            :  out   std_logic_vector((data_width-1) downto 0);   -- Upsampled cr output
      cb_out            :  out   std_logic_vector((data_width-1) downto 0);   -- Upsampled cb output
      saturate          :  out   std_logic;
      sat_type          :  out   std_logic;
      dout_valid        :  out   std_logic                                    -- Depends upon latency through system
      
   );
end convert422to444;

architecture rtl of convert422to444 is

-- Here we ensure that the number of taps implemented is an even number - this is better for this architecture
constant num_h_taps_div2            :  integer  := (num_taps+1)/2; -- 2->1; 3->2; 4->2; 5->3 ...
constant numhtaps_made_even         :  integer  := (num_h_taps_div2 * 2); -- eg if num_taps=3 then we get 4 taps

constant latency                    :  integer := ((1-replicate)*(3*(numhtaps_made_even/2) + (numhtaps_made_even/2) + 1 + 1)) + (replicate*6); -- 2 taps: use 6; 4 taps: use 10
type     pipe_type                  is array ((4*numhtaps_made_even) downto 0) of std_logic_vector((data_width - 1) downto 0);
signal   pipe                       :  pipe_type;
type     mult_in_type               is array ((numhtaps_made_even - 1)downto 0) of std_logic_vector(data_width downto 0);
signal   mult_in                    :  mult_in_type;
type     mult_out_type              is array ((numhtaps_made_even - 1)downto 0) of std_logic_vector((coef_width + data_width) downto 0);
signal   mult_out                   :  mult_out_type;
type     coefs_array_type           is array (num_taps downto 0) of std_logic_vector((coef_width - 1) downto 0);
signal   coefs_array                :  coefs_array_type;
signal   d_cr_active                 :  std_logic_vector((4*numhtaps_made_even) downto 0);    -- Active High indicates u sample on input
type     d_luma_type                is array (latency downto 0) of std_logic_vector((data_width - 1) downto 0);
signal   d_luma                     :  d_luma_type;
signal   d_data_valid               :  std_logic_vector(latency downto 0);
signal   d_chroma_in                :  std_logic_vector((data_width - 1) downto 0);
type     mult_out_int_type          is array (0 to (numhtaps_made_even - 1)) of integer;
signal   mult_out_int               :  mult_out_int_type;
type     partial_sum_int_type       is array ((numhtaps_made_even - 1) downto 0) of integer;
signal   partial_sum_int            :  partial_sum_int_type;
signal   rounded_partial_sum        :  integer;
signal   replicated_pixel           :  std_logic_vector((data_width - 1) downto 0);
signal   cr_out_int, cb_out_int     :  integer;
constant full_precision_width       :  integer := data_width + coef_width + LOG2_BASE(numhtaps_made_even);
signal   chroma_out_full_precision  :  std_logic_vector((full_precision_width - 1) downto 0);
signal   rounding_offset            :  integer := 2**(coef_width - 2);
signal   truncated_chroma_out       :  std_logic_vector((data_width - 1) downto 0);
signal   early_cr_out               :  std_logic_vector((data_width - 1) downto 0);
signal   vs_out                                    :  std_logic;
signal   early_sat                  :  std_logic;
signal   early_cr_sat               :  std_logic;
signal   early_sat_type             :  std_logic;
signal   early_cr_sat_type          :  std_logic;
begin

-- Coefs are fed in from GUI as integers. Convert them to std_logic_vectors here.
convert_coefs : for i in 0 to (num_taps - 1) generate
   coefs_array(i)    <= conv_std_logic_vector(coefs(i), coef_width);
end generate;
coefs_array(num_taps)    <= (others => '0');

mult_in_gen : for i in 0 to (numhtaps_made_even - 1) generate
   mult_in(i)  <= ('0' & pipe(3*i));
end generate;

clk_process: process(clk)
begin
   if (clk'event and clk = '1') then
      if (ce = '1') then
      
      -- Register Luma_in, luma_out, chroma_in and data-valid signals - this counts as first and last delays.         
         d_luma(0)         <= luma_in;
         luma_out          <= d_luma(latency);   
         d_data_valid(0)   <= din_valid;
         dout_valid        <= d_data_valid(latency);
         d_chroma_in       <= chroma_in;
     
      -- Create flag indicating Chroma sample type.
         if (din_valid = '0') then
            d_cr_active(0)    <= '1'; -- Start with Cr sample 
         else
            d_cr_active(0)    <= not (d_cr_active(0));
         end if;
   
         delay_cr_active : for i in 1 to (4*numhtaps_made_even) loop
            d_cr_active(i)     <= d_cr_active(i-1);
         end loop;
   
      -- Define contents of pipeline.
         -- Now shift new value into front location remembering inactive video value.
         if (din_valid = '0') then
            pipe(0)     <= conv_std_logic_vector(pad_val, data_width);
         else
            pipe(0)     <= chroma_in;
         end if;      
         
         -- Shift values along pipe
         -- Cr and Cb samples are multiplexed on the input, so we make the pipeline longer,
         -- including pipeline registers
         define_taps    : for i in 1 to (4*numhtaps_made_even) loop
            pipe(i)     <= pipe(i-1);
         end loop;
   
      -- Multiply taps by appropriate coefficients
         mult_loop : for i in 0 to (numhtaps_made_even - 1) loop
            mult_out(i) <= ('0' & pipe(3*i)) * coefs_array(i);
         end loop;
   
         -- Here is the final adder chain (registered).
         partial_sum_int(0)       <= mult_out_int(0);
         add_loop :  for i in 0 to (numhtaps_made_even - 2) loop
            partial_sum_int(i+1)  <= partial_sum_int(i) + mult_out_int(i+1);
         end loop;
      
         rounded_partial_sum     <= partial_sum_int(numhtaps_made_even - 1) + rounding_offset;
         if (replicate = 1) then
            replicated_pixel        <= pipe(4);
         else
            replicated_pixel        <= pipe(2*numhtaps_made_even);
         end if;
   
      -- MUX outputs - also register for chroma outputs
         -- When 'replicate' is set to 1, the centre-tap is routed directly to the output. This 'should'
         -- optimise the rest of the filter out......
         if (d_cr_active((3*(numhtaps_made_even/2)) + (numhtaps_made_even/2) + 1 + 1) = '1') then -- 2-taps - use 6; 4-taps use
            early_cr_out      <= replicated_pixel;
         elsif (replicate = 0) then
            early_cr_out      <= truncated_chroma_out;
            early_cr_sat      <= early_sat;
            early_cr_sat_type <= early_sat_type;
         end if;
         
   
         if (d_cr_active((3*(numhtaps_made_even/2)) + (numhtaps_made_even/2) + 1 + 1) = '0') then
            cb_out            <= replicated_pixel;
         elsif (replicate = 0) then
            cb_out            <= truncated_chroma_out;
         end if;
   
         -- Cr_out requires one more clock-cycle delay than cb_out
         cr_out               <= early_cr_out;
         saturate             <= early_sat or early_cr_sat;
         sat_type             <= early_sat_type or early_cr_sat_type;
   
      -- Delay Luma and data-valid signal
         compensate_delay : for i in 1 to latency loop
            d_luma(i)         <= d_luma(i-1);
            d_data_valid(i)   <= d_data_valid(i-1);
         end loop;
         
      end if;
   end if;
end process;

-- Asynchronous assignments
   -- Prior to adding the multiply results, convert all mult out values to integers.
   -- This makes generic coding much simpler since you don't have to deal with all those nasty sign extensions.
   convert_mult_out : for i in 0 to (numhtaps_made_even - 1) generate
      mult_out_int(i)    <= conv_integer(mult_out(i));
   end generate;
   
   chroma_out_full_precision        <= conv_std_logic_vector(rounded_partial_sum, full_precision_width);
   truncated_chroma_out             <= chroma_out_full_precision((coef_width + data_width-2) downto (coef_width-1));

   early_sat                        <= chroma_out_full_precision(coef_width + data_width-1) when replicate = 0 else '0';
   early_sat_type                   <= chroma_out_full_precision(coef_width + data_width) when replicate = 0 else '0';         

   
end rtl;