LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
use ieee.std_logic_arith.all;
use ieee.std_logic_signed.all; 
use work.ImageXlib_utils.all;

entity convert422to444_wrap is
generic
(
      num_taps                :  integer        := 2;          -- coefs must be an array of at least this length. Extra values in array are ignored.
      data_width              :  integer        := 8;       
      coef_width              :  integer        := 2;          -- All integer coefficients must add up to 2^(coef_width - 1)
      pad_val                 :  integer        := 0;          -- Value used to fill pipeline at image edges - currently same for cr and cb
      replicate               :  integer        := 0;          -- If 1, then filter should be optimised out
      coefs	                  :	INTEGER_ARRAY  := (1, 1)      -- Parameterised, but not dynamic.
);
port
(
      ce                      :  in    std_logic;
      clk                     :  in    std_logic;
      din_valid               :  in    std_logic;
      vsync_in                :  in    std_logic;
      hsync_in                :  in    std_logic;
      vblank_in               :  in    std_logic;
      hblank_in               :  in    std_logic;
      luma_in                 :  in    std_logic_vector((data_width-1) downto 0);   -- Brought in for latency equalisation
      chroma_in               :  in    std_logic_vector((data_width-1) downto 0);   -- Multiplexed cr, cb
      luma_out                :  out   std_logic_vector((data_width-1) downto 0);   -- Latency equalised luma output
      cr_out                  :  out   std_logic_vector((data_width-1) downto 0);   -- Upsampled cr output
      cb_out                  :  out   std_logic_vector((data_width-1) downto 0);   -- Upsampled cb output
      vsync_out               :  out   std_logic;
      hsync_out               :  out   std_logic;
      vblank_out              :  out   std_logic;
      hblank_out              :  out   std_logic;
      saturate                :  out   std_logic;
      sat_type                :  out   std_logic;
      dout_valid              :  out   std_logic                                    -- Depends upon latency through system
   
);
end convert422to444_wrap;

architecture rtl of convert422to444_wrap is

signal   left_cr_pad                :  std_logic_vector((data_width - 1) downto 0);
signal   left_cb_pad                :  std_logic_vector((data_width - 1) downto 0);
signal   right_cr_pad               :  std_logic_vector((data_width - 1) downto 0);
signal   right_cb_pad               :  std_logic_vector((data_width - 1) downto 0);
type     DelayedVChanType           is array (31 downto 0) of std_logic_vector((data_width - 1) downto 0);
signal   DelayedChromaIn            :  DelayedVChanType;
signal   DelayedYIn                 :  DelayedVChanType;


signal   Delayed_din_valid          :  std_logic_vector(31 downto 0);
signal   Delayed_vsync              :  std_logic_vector(31 downto 0);
signal   Delayed_hsync              :  std_logic_vector(31 downto 0);
signal   Delayed_vblank             :  std_logic_vector(31 downto 0);
signal   Delayed_hblank             :  std_logic_vector(31 downto 0);
signal   d_chroma_in                :  std_logic_vector((data_width - 1) downto 0);
signal   d_y_in                     :  std_logic_vector((data_width - 1) downto 0);
signal   d_din_valid                :  std_logic;
signal   pad_sel                    :  std_logic;

begin


proc1 : process (clk)
begin
if (clk'event) and (clk = '1') then
   if   (ce = '1') then
      DelayedChromaIn(0)               <= chroma_in;
      DelayedChromaIn(31 downto 1)     <= DelayedChromaIn(30 downto 0); -- sufficiently big for future generic case - should be optimised out.

      Delayed_din_valid(0)             <= din_valid;
      Delayed_din_valid(31 downto 1)   <= Delayed_din_valid(30 downto 0); -- sufficiently big for future generic case - should be optimised out.
      d_din_valid                      <= Delayed_din_valid(0) or Delayed_din_valid(4);   
                                                   -- din_valid must be lengthened by a period equal to the length of the filter.

      DelayedYIn(0)                    <= luma_in;
      DelayedYIn(31  downto 1)         <= DelayedYIn(30 downto 0); -- sufficiently big for future generic case - should be optimised out.
      d_y_in                           <= DelayedYIn(2); -- Tap off delay line at a point equal to half the filter size.

      Delayed_vsync(0)                 <= vsync_in;
      Delayed_vsync(31 downto 1)       <= Delayed_vsync(30 downto 0); -- sufficiently big for future generic case - should be optimised out.
      vsync_out                        <= Delayed_vsync(10); 

      Delayed_hsync(0)                 <= hsync_in;
      Delayed_hsync(31 downto 1)       <= Delayed_hsync(30 downto 0); -- sufficiently big for future generic case - should be optimised out.
      hsync_out                        <= Delayed_hsync(10); 

      Delayed_vblank(0)                <= vblank_in;
      Delayed_vblank(31 downto 1)      <= Delayed_vblank(30 downto 0); -- sufficiently big for future generic case - should be optimised out.
      vblank_out                       <= Delayed_vblank(10); 

      Delayed_hblank(0)                <= hblank_in;
      Delayed_hblank(31 downto 1)      <= Delayed_hblank(30 downto 0); -- sufficiently big for future generic case - should be optimised out.
      hblank_out                       <= Delayed_hblank(10); 

      dout_valid                       <= Delayed_din_valid(10);

      -- Detect rising (left) edge of din_valid and use Cr and Cb input values to pad filter
      -- before first data on current line enters the filter
      if (din_valid = '1') and (Delayed_din_valid(0) = '0') then
         left_cr_pad    <= chroma_in;
      end if;
      -- Note 1st Cb sample occurs one cycle after 1st Cr sample
      if (Delayed_din_valid(0) = '1') and (Delayed_din_valid(1) = '0') then
         left_cb_pad    <= chroma_in;
      end if;
   
      -- Create selector between Cr and Cb pad value
      if (din_valid = '0') and (Delayed_din_valid(8) = '0') then
         pad_sel  <= '0';
      else
         pad_sel  <= not(pad_sel);
      end if;
      
      -- Detect falling (right) edge of din_valid and use Cr and Cb input values to pad filter
      -- after last data on current line enters filter
      if (din_valid = '0') and (Delayed_din_valid(0) = '1') then
         right_cb_pad   <= DelayedChromaIn(0);
         right_cr_pad   <= DelayedChromaIn(1);
      end if;
      
      -- Mux delayed input data with padding values
      if (Delayed_din_valid(0)  =  '1') and (Delayed_din_valid(3) = '0') then
         --   Left edge
         if (pad_sel = '0') then
            d_chroma_in                <= left_cb_pad;
         else
            d_chroma_in                <= left_cr_pad;
         end if;            
      elsif (Delayed_din_valid(2) = '0') and (Delayed_din_valid(4) = '1')  then
         --   Right edge
         if (pad_sel = '1') then
            d_chroma_in                <= right_cr_pad;
         else
            d_chroma_in                <= right_cb_pad;
         end if;
      elsif (Delayed_din_valid(0) = '0') and (Delayed_din_valid(2) = '0') then
         d_chroma_in                   <= (others => '0');
         d_chroma_in                   <= (others => '0');
      else
          -- Everywhere else
         d_chroma_in                   <= DelayedChromaIn(2);
      end if;
      
   end if;
end if;
end process;

u0 : entity work.convert422to444
generic map
(
   num_taps       => num_taps,
   data_width     => data_width, 
   coef_width     => coef_width, 
   pad_val        => pad_val,
   replicate      => replicate, 
   coefs	         => coefs    
)
port map
(                 
   clk            => clk,
   ce             => ce,
   din_valid      => d_din_valid, -- Select correct delay tap according to filter size
   luma_in        => d_y_in,
   chroma_in      => d_chroma_in,
   luma_out       => luma_out,
   cr_out         => cr_out,
   cb_out         => cb_out,
   saturate       => saturate,
   sat_type       => sat_type,
   dout_valid     => open
);

end rtl;
