------------------------------------------------------------------
--      _____
--     /     \
--    /____   \____
--   / \===\   \==/
--  /___\===\___\/  AVNET
--       \======/
--        \====/    
-----------------------------------------------------------------
--
-- This design is the property of Avnet.  Publication of this
-- design is not authorized without written consent from Avnet.
-- 
-- Please direct any questions to:  technical.support@avnet.com
--
-- Disclaimer:
--    Avnet, Inc. makes no warranty for the use of this code or design.
--    This code is provided  "As Is". Avnet, Inc assumes no responsibility for
--    any errors, which may appear in this code, nor does it make a commitment
--    to update the information contained herein. Avnet, Inc specifically
--    disclaims any implied warranties of fitness for a particular purpose.
--                     Copyright(c) 2010 Avnet, Inc.
--                             All rights reserved.
--
------------------------------------------------------------------
--
-- Create Date:         Apr 13, 2010
-- Design Name:         Color Space Converter - YCrCb 4:2:2 to RGB
-- Module Name:         csc_ycrcb422_to_rgb.vhd
-- Project Name:        IVK
-- Target Devices:      Spartan-6
-- Avnet Boards:        IVK
--
-- Tool versions:       ISE 11.4
--
-- Description:         This module performs two conversion steps:
--                      1) Chroma Resampling
--                         - YCrCb 4:2:2 to YCrCb 4:4:4
--                         - based on XAPP932
--                      2) Color Space Conversion
--                         - YCrCB 4:4:4 to RGB
--                         - based on XAPP931
--
--                      The Video I/O for this module are:
--                      - Video Output = 24 bit RGB
--                      - Video Input  = 16 bit YCrCb 4:2:2
--
-- References:        
--    http://www.xilinx.com/support/documentation/application_notes/xapp930.pdf
--    http://www.xilinx.com/support/documentation/application_notes/xapp931.pdf
--    http://www.xilinx.com/support/documentation/application_notes/xapp932.pdf
--                      
-- Dependencies:        
--
-- Revision:            Apr 13, 2010: 1.01 Initial version
--
------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

use WORK.ImageXlib_utils.all;

entity csc_ycrcb422_to_rgb is
   port (
      clk               : in  STD_LOGIC;
      ce                : in  STD_LOGIC;

      -- XSVI Input (16 bit YCrCb 4:2:2)
      vblank_i          : in  STD_LOGIC;
      hblank_i          : in  STD_LOGIC;
      vsync_i           : in  STD_LOGIC;
      hsync_i           : in  STD_LOGIC;
      active_video_i	: in  STD_LOGIC;
      video_data_i      : in  STD_LOGIC_VECTOR (15 downto 0);

      -- XSVI Input (24 bit RGB)
      vblank_o          : out STD_LOGIC;
      hblank_o          : out STD_LOGIC;
      vsync_o           : out STD_LOGIC;
      hsync_o           : out STD_LOGIC;
      active_video_o    : out STD_LOGIC;
      video_data_o      : out STD_LOGIC_VECTOR (23 downto 0);

      debug_o           : out STD_LOGIC_VECTOR (26 downto 0)
           
    );
end csc_ycrcb422_to_rgb;


architecture rtl of csc_ycrcb422_to_rgb is

   signal vblank_d      : STD_LOGIC;
   signal hblank_d      : STD_LOGIC;
   signal vsync_d       : STD_LOGIC;
   signal hsync_d       : STD_LOGIC;
   signal active_video_d: STD_LOGIC;
   signal video_luma_d  : STD_LOGIC_VECTOR (7 downto 0);
   signal video_chroma_d: STD_LOGIC_VECTOR (7 downto 0);

   --
   -- Chroma Re-Sampling (16 bit YCrCb 4:2:2 to 24 bit YCrCb 4:4:4)
   --

   component convert422to444_wrap
   generic
   (
      num_taps                :  integer        := 2;          -- coefs must be an array of at least this length. Extra values in array are ignored.
      data_width              :  integer        := 8;       
      coef_width              :  integer        := 2;          -- All integer coefficients must add up to 2^(coef_width - 1)
      pad_val                 :  integer        := 0;          -- Value used to fill pipeline at image edges - currently same for cr and cb
      replicate               :  integer        := 0;          -- If 1, then filter should be optimised out
      coefs	                  :	INTEGER_ARRAY  := (1, 1)      -- Parameterised, but not dynamic.
   );
   port
   (
      ce                      :  in    std_logic;
      clk                     :  in    std_logic;
      din_valid               :  in    std_logic;
      vblank_in               :  in    std_logic;
      hblank_in               :  in    std_logic;
      vsync_in                :  in    std_logic;
      hsync_in                :  in    std_logic;
      luma_in                 :  in    std_logic_vector((data_width-1) downto 0);   -- Brought in for latency equalisation
      chroma_in               :  in    std_logic_vector((data_width-1) downto 0);   -- Multiplexed cr, cb
      luma_out                :  out   std_logic_vector((data_width-1) downto 0);   -- Latency equalised luma output
      cr_out                  :  out   std_logic_vector((data_width-1) downto 0);   -- Upsampled cr output
      cb_out                  :  out   std_logic_vector((data_width-1) downto 0);   -- Upsampled cb output
      vblank_out              :  out   std_logic;
      hblank_out              :  out   std_logic;
      vsync_out               :  out   std_logic;
      hsync_out               :  out   std_logic;
      saturate                :  out   std_logic;
      sat_type                :  out   std_logic;
      dout_valid              :  out   std_logic                                    -- Depends upon latency through system
   );
   end component;

   signal   t_Y444      :  std_logic_vector(7 downto 0);
   signal   t_Cr444     :  std_logic_vector(7 downto 0);
   signal   t_Cb444     :  std_logic_vector(7 downto 0);
   signal   t_de_o      :  std_logic;
   signal   t_vsync_o   :  std_logic;
   signal   t_hsync_o   :  std_logic;
   signal   t_vblank_o  :  std_logic;
   signal   t_hblank_o  :  std_logic;

   --
   -- Color Space Conversion (24 bit YCrCb to 24 bit RGB)
   --

   component Xil_YCrCb2RGB is   
   generic
   ( 
      FAMILY_HAS_MAC: integer:= 1;
      FABRIC_ADDS   : integer:= 1; -- Adders are implemented using logic fabric based adders
      IWIDTH        : integer:= 9;
      CWIDTH        : integer:= 13; -- Coefficients are signed, CWIDTH.CWIDTH-2 format
      MWIDTH        : integer:= 23; -- ONLY FOR NON-V4: Controls bits witheld after mults. 
      OWIDTH        : integer:= 9;  -- OTHERWISE (default) IWIDTH+CWIDTH+1;
      RGBMAX        : integer:= 255;
      RGBMIN        : integer:= 0;
      ACOEF         : integer:= 2872;   --    1.4023   *pow2(CWIDTH-2) 
      BCOEF         : integer:= -1461;  --    -0.7133  *pow2(CWIDTH-2) 
      CCOEF         : integer:= -703;   --    -0.3434  *pow2(CWIDTH-2) 
      DCOEF         : integer:= 3630;   --    1.7724   *pow2(CWIDTH-2) 
      ROFFSET       : integer:= -366592;  -- Should be MWIDTH bits wide 
      GOFFSET       : integer:= 278016;   
      BOFFSET       : integer:= -463616;
      HAS_CLIP      : integer:= 1; 
      HAS_CLAMP     : integer:= 1
   );     
   port
   (                               
      Y             : in std_logic_vector(IWIDTH-1 downto 0);  -- Y  = a(R-G) + G + b(B-G)
      Cr            : in std_logic_vector(IWIDTH-1 downto 0);  -- Cr = d(R-Y)  
      Cb            : in std_logic_vector(IWIDTH-1 downto 0);  -- Cb = c(B-Y)
      R             : out std_logic_vector(OWIDTH-1 downto 0);   
      G             : out std_logic_vector(OWIDTH-1 downto 0);
      B             : out std_logic_vector(OWIDTH-1 downto 0);
      V_BLANK_in    : in std_logic := '0';
      H_BLANK_in    : in std_logic := '0';
      V_SYNC_in     : in std_logic := '0';
      H_SYNC_in     : in std_logic := '0';
      PIX_EN_in     : in std_logic := '1';
      V_BLANK_out   : out std_logic;
      H_BLANK_out   : out std_logic;
      V_SYNC_out    : out std_logic;
      H_SYNC_out    : out std_logic;
      PIX_EN_out    : out std_logic;
      clk           : in std_logic;
      ce            : in std_logic := '1';
      sclr          : in std_logic := '0'
   );
   end component;

   signal   R_YCrCbtoRGB       : std_logic_vector(7 downto 0);
   signal   G_YCrCbtoRGB       : std_logic_vector(7 downto 0);
   signal   B_YCrCbtoRGB       : std_logic_vector(7 downto 0);
   signal   vblank_YCrCbtoRGB  : std_logic;
   signal   hblank_YCrCbtoRGB  : std_logic;
   signal   vsync_YCrCbtoRGB   : std_logic;
   signal   hsync_YCrCbtoRGB   : std_logic;
   signal   de_YCrCbtoRGB      : std_logic;
   

begin

   --
   -- Input Registers
   --

   In_Reg : process (clk)
   begin
      if Rising_Edge(clk) then
         vblank_d       <= vblank_i;
         hblank_d       <= hblank_i;
         vsync_d        <= vsync_i;
         hsync_d        <= hsync_i;
         active_video_d <= active_video_i;
         video_luma_d   <= video_data_i( 7 downto  0);
         video_chroma_d <= video_data_i(15 downto  8);
      end if;
   end process;

   --
   -- Chroma Re-Sampling (16 bit YCrCb 4:2:2 to 24 bit YCrCb 4:4:4)
   --

   YCrCb422to444: convert422to444_wrap
   generic map
   (
      num_taps                => 2,          -- coefs must be an array of at least this length. Extra values in array are ignored.
      data_width              => 8,       
      coef_width              => 2,          -- All integer coefficients must add up to 2^(coef_width - 1)
      pad_val                 => 0,          -- Value used to fill pipeline at image edges - currently same for cr and cb
      replicate               => 0,          -- If 1, then filter should be optimised out
      coefs	                  => (1, 1)      -- Parameterised, but not dynamic.
   )
   port map
   (
      ce                      => '1',
      clk                     => clk,
      din_valid               => active_video_d,
      vblank_in               => vblank_d,
      hblank_in               => hblank_d,
      vsync_in                => vsync_d,
      hsync_in                => hsync_d,
      luma_in                 => video_luma_d,     -- Brought in for latency equalisation
      chroma_in               => video_chroma_d,   -- Multiplexed cr, cb
      luma_out                => t_Y444,           -- Latency equalised luma output
      cr_out                  => t_Cr444,          -- Upsampled cr output
      cb_out                  => t_Cb444,          -- Upsampled cb output
      vblank_out              => t_vblank_o,
      hblank_out              => t_hblank_o,
      vsync_out               => t_vsync_o,
      hsync_out               => t_hsync_o,
      saturate                => open,
      sat_type                => open,
      dout_valid              => t_de_o            -- Depends upon latency through system
   );

   --
   -- Color Space Conversion (24 bit YCrCb to 24 bit RGB)
   --

   YCrCb2RGB: Xil_YCrCb2RGB
   generic map
   ( 
      FAMILY_HAS_MAC           => 1,
      FABRIC_ADDS              => 1,        -- Adders are implemented using logic fabric based adders
      IWIDTH                   => 8,
      CWIDTH                   => 13,       -- Coefficients are signed, CWIDTH.CWIDTH-2 format
      MWIDTH                   => 22,       -- ONLY FOR NON-V4: Controls bits witheld after mults. 
      OWIDTH                   => 8,        -- OTHERWISE (default) IWIDTH+CWIDTH+1
      RGBMAX                   => 255,
      RGBMIN                   => 0,
      ACOEF                    => 2872,     --    1.4023   *pow2(CWIDTH-2) 
      BCOEF                    => -1461,    --    -0.7133  *pow2(CWIDTH-2) 
      CCOEF                    => -703,     --    -0.3434  *pow2(CWIDTH-2) 
      DCOEF                    => 3630,     --    1.7724   *pow2(CWIDTH-2) 
      ROFFSET                  => -366592,  -- Should be MWIDTH bits wide 
      GOFFSET                  => 278016,   
      BOFFSET                  => -463616,
      HAS_CLIP                 => 1, 
      HAS_CLAMP                => 1
   )   
   port map
   (                               
      Y                        => t_Y444,   -- Y  = a(R-G) + G + b(B-G)
      Cr                       => t_Cr444,  -- Cr = d(R-Y)  
      Cb                       => t_Cb444,  -- Cb = c(B-Y)
      R                        => R_YCrCbtoRGB,   
      G                        => G_YCrCbtoRGB,
      B                        => B_YCrCbtoRGB,
      V_BLANK_in               => t_vblank_o,
      H_BLANK_in               => t_hblank_o,
      V_SYNC_in                => t_vsync_o,
      H_SYNC_in                => t_hsync_o,
      PIX_EN_in                => t_de_o,
      V_BLANK_out              => vblank_YCrCbtoRGB,
      H_BLANK_out              => hblank_YCrCbtoRGB,
      V_SYNC_out               => vsync_YCrCbtoRGB,
      H_SYNC_out               => hsync_YCrCbtoRGB,
      PIX_EN_out               => de_YCrCbtoRGB,
      clk                      => clk,
      ce                       => '1',
      sclr                     => '0'
   );

   --
   -- Output Registers
   --

   Out_Reg : process (clk)
   begin
      if Rising_Edge(clk) then
         vblank_o          <= vblank_YCrCbtoRGB;
         hblank_o          <= hblank_YCrCbtoRGB;
         vsync_o           <= vsync_YCrCbtoRGB;
         hsync_o           <= hsync_YCrCbtoRGB;
         active_video_o    <= de_YCrCbtoRGB;
         video_data_o      <= R_YCrCbtoRGB & B_YCrCbtoRGB & G_YCrCbtoRGB;
      end if;
   end process;

   --
   -- Debug Port
   --

   Debug_Port : process (clk)
   begin
      if Rising_Edge(clk) then
         debug_o  <= t_de_o & t_vsync_o & t_hsync_o & t_Y444 & t_Cr444 & t_Cb444;
      end if;
   end process;

end rtl;
