/* $Id: */
/******************************************************************************
* (c) Copyright 2009 Xilinx, Inc. All rights reserved.
*
* This file contains confidential and proprietary information
* of Xilinx, Inc. and is protected under U.S. and
* international copyright and other intellectual property
* laws.
*
* DISCLAIMER
* This disclaimer is not a license and does not grant any
* rights to the materials distributed herewith. Except as
* otherwise provided in a valid license issued to you by
* Xilinx, and to the maximum extent permitted by applicable
* law: (1) THESE MATERIALS ARE MADE AVAILABLE "AS IS" AND
* WITH ALL FAULTS, AND XILINX HEREBY DISCLAIMS ALL WARRANTIES
* AND CONDITIONS, EXPRESS, IMPLIED, OR STATUTORY, INCLUDING
* BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, NON-
* INFRINGEMENT, OR FITNESS FOR ANY PARTICULAR PURPOSE; and
* (2) Xilinx shall not be liable (whether in contract or tort,
* including negligence, or under any other theory of
* liability) for any loss or damage of any kind or nature
* related to, arising under or in connection with these
* materials, including for any direct, or any indirect,
* special, incidental, or consequential loss or damage
* (including loss of data, profits, goodwill, or any type of
* loss or damage suffered as a result of any action brought
* by a third party) even if such damage or loss was
* reasonably foreseeable or Xilinx had been advised of the
* possibility of the same.
*
* CRITICAL APPLICATIONS
* Xilinx products are not designed or intended to be fail-
* safe, or for use in any application requiring fail-safe
* performance, such as life-support or safety devices or
* systems, Class III medical devices, nuclear facilities,
* applications related to the deployment of airbags, or any
* other applications that could lead to death, personal
* injury, or severe property or environmental damage
* (individually and collectively, "Critical
* Applications"). Customer assumes the sole risk and
* liability of any use of Xilinx products in Critical
* Applications, subject only to applicable laws and
* regulations governing limitations on product liability.
*
* THIS COPYRIGHT NOTICE AND DISCLAIMER MUST BE RETAINED AS
* PART OF THIS FILE AT ALL TIMES.
*
* All rights reserved.
*
******************************************************************************/
/*****************************************************************************/
/**
*
* @file xtimebase_intr.c
*
* This code contains interrupt related functions of Xilinx MVI Video TimeBase
* device driver. Please see xtimebase.h for more details of the driver.
*
* <pre>
* MODIFICATION HISTORY:
*
* Ver	Who	Date		Changes
* -----	----	--------	-----------------------------------------------
* 1.00a	xd	08/05/08	First release
* 1.01a	xd	07/23/10	Added GIER; Added more h/w generic info into
*				xparameters.h; Feed callbacks with pending
*				interrupt info. Added Doxygen & Version support
* </pre>
*
******************************************************************************/

#include "xtimebase.h"
#include "xil_assert.h"
#include "xil_types.h"

/*****************************************************************************/
/**
*
* This function is the interrupt handler for the TimeBase driver.
*
* This handler reads the pending interrupt from the IER/ISR, determines the
* source of the interrupts, calls according callbacks, and finally clears the
* interrupts.
*
* The application is responsible for connecting this function to the interrupt
* system. Application beyond this driver is also responsible for providing
* callbacks to	handle interrupts and installing the callbacks using
* XTimeBase_SetCallBack() during initialization phase. An example delivered
* with this driver demonstrates how this could be done.
*
* @param   InstancePtr is a pointer to the XTimeBase instance that just
*	   interrupted.
* @return  None.
* @note	   None.
*
******************************************************************************/
void XTimeBase_IntrHandler(void *InstancePtr)
{
	u32 PendingIntr;
	u32 ErrorStatus;
	XTimeBase *XTimeBasePtr = (XTimeBase *) InstancePtr;

	/* Validate parameters */
	Xil_AssertVoid(XTimeBasePtr != NULL);
	Xil_AssertVoid(XTimeBasePtr->IsReady == XIL_COMPONENT_IS_READY);

	/* Get pending interrupts */
	PendingIntr = XTimeBase_IntrGetPending(XTimeBasePtr);

	/* Clear pending interrupt(s) */
	XTimeBase_IntrClear(XTimeBasePtr, PendingIntr);

	/* Spurious interrupt has happened */
	if (0 == (PendingIntr | XTB_IXR_ALLINTR_MASK)) {
		ErrorStatus = 0;
		XTimeBasePtr->ErrCallBack(XTimeBasePtr->ErrRef, ErrorStatus);
		return;
	}

	/* A generator event has happened */
	if ((PendingIntr & XTB_IXR_G_ALL_MASK))
		XTimeBasePtr->GeneratorCallBack(XTimeBasePtr->GeneratorRef,
		PendingIntr);

	/* A detector event has happened */
	if ((PendingIntr & XTB_IXR_D_ALL_MASK))
		XTimeBasePtr->DetectorCallBack(XTimeBasePtr->DetectorRef,
		PendingIntr);

	/* A frame sync is done */
	if ((PendingIntr & XTB_IXR_FSYNCALL_MASK))
		XTimeBasePtr->FrameSyncCallBack(XTimeBasePtr->FrameSyncRef,
		PendingIntr);

	/* A signal lock is detected */
	if ((PendingIntr & XTB_IXR_LOCKALL_MASK))
		XTimeBasePtr->LockCallBack(XTimeBasePtr->LockRef,
		PendingIntr);
}


/*****************************************************************************/
/**
*
* This routine installs an asynchronous callback function for the given
* HandlerType:
*
* <pre>
* HandlerType		   Callback Function Type
* -----------------------  ---------------------------
* XTB_HANDLER_FRAMESYNC	   XTimeBase_FrameSyncCallBack
* XTB_HANDLER_LOCK	   XTimeBase_LockCallBack
* XTB_HANDLER_DETECTOR	   XTimeBase_DetectorCallBack
* XTB_HANDLER_GENERATOR	   XTimeBase_GeneratorCallBack
* XTB_HANDLER_ERROR	   XTimeBase_ErrCallBack
*
* HandlerType		   Invoked by this driver when:
* -----------------------  --------------------------------------------------
* XTB_HANDLER_FRAMESYNC	   A frame sync event happens
* XTB_HANDLER_LOCK	   A signal lock event happens
* XTB_HANDLER_DETECTOR	   A detector related event happens
* XTB_HANDLER_GENERATOR	   A generator related event happens
* XTB_HANDLER_ERROR	   An error condition happens
*
* </pre>
*
* @param	InstancePtr is a pointer to the XTimeBase instance to be worked
*               on.
* @param	HandlerType specifies which callback is to be attached.
* @param	CallbackFunc is the address of the callback function.
* @param	CallbackRef is a user data item that will be passed to the
*		callback function when it is invoked.
*
* @return
*		- XST_SUCCESS when handler is installed.
*		- XST_INVALID_PARAM when HandlerType is invalid.
*
* @note
* Invoking this function for a handler that already has been installed replaces
* it with the new handler.
*
******************************************************************************/
int XTimeBase_SetCallBack(XTimeBase *InstancePtr, u32 HandlerType,
				void *CallBackFunc, void *CallBackRef)
{
	Xil_AssertNonvoid(InstancePtr != NULL);
	Xil_AssertNonvoid(InstancePtr->IsReady == XIL_COMPONENT_IS_READY);

	switch (HandlerType) {
	case XTB_HANDLER_FRAMESYNC:
		InstancePtr->FrameSyncCallBack =
				(XTimeBase_CallBack) CallBackFunc;
		InstancePtr->FrameSyncRef = CallBackRef;
		break;

	case XTB_HANDLER_LOCK:
		InstancePtr->LockCallBack = (XTimeBase_CallBack) CallBackFunc;
		InstancePtr->LockRef = CallBackRef;
		break;

	case XTB_HANDLER_DETECTOR:
		InstancePtr->DetectorCallBack =
				(XTimeBase_CallBack) CallBackFunc;
		InstancePtr->DetectorRef = CallBackRef;
		break;

	case XTB_HANDLER_GENERATOR:
		InstancePtr->GeneratorCallBack =
				(XTimeBase_CallBack) CallBackFunc;
		InstancePtr->GeneratorRef = CallBackRef;
		break;

	case XTB_HANDLER_ERROR:
		InstancePtr->ErrCallBack =
				(XTimeBase_ErrorCallBack) CallBackFunc;
		InstancePtr->ErrRef = CallBackRef;
		break;

	default:
		return XST_INVALID_PARAM;

	}
	return XST_SUCCESS;
}
